@@ -98,7 +98,7 @@ pub mod repository;
 
 pub use cursor::ModelCursor;
 pub use index::{sync_indexes, Index, IndexOption, Indexes, SortOrder};
-pub use repository::Repository;
+pub use repository::{BulkUpdate, BulkUpdateResult, BulkUpdateUpsertResult, Repository};
 
 // Re-export mongodb
 pub use mongodb as mongo;
@@ -172,8 +172,11 @@ impl ToRepository for mongodb::Database {
 pub mod prelude {
     #[doc(no_inline)]
     pub use crate::mongo::bson::{
-        bson, de::Error as BsonDeError, doc, from_bson, oid::ObjectId, ser::Error as BsonSerError,
-        to_bson, Bson, DateTime as BsonDateTime, Document as BsonDocument,
+        bson, doc, from_bson, to_bson,
+        Bson, oid::ObjectId, ser::Error as BsonSerError, de::Error as BsonDeError,
+        DateTime as BsonDateTime, Document as BsonDocument, Timestamp as BsonTimestamp,
+        Binary as BsonBinary, JavaScriptCodeWithScope as BsonJavaScriptCodeWithScope,
+        Regex as BsonRegex, Serializer as BsonSerializer, Deserializer as BsonDeserializer,
     };
     #[doc(no_inline)]
     pub use crate::mongo::{
@@ -223,8 +226,9 @@ pub mod prelude {
     pub use crate::ToRepository;
     #[doc(no_inline)]
     pub use crate::{
-        f, field, mongo::Cursor, operator::*, pipeline, sync_indexes, CollectionConfig, Index,
-        IndexOption, Indexes, Model, ModelCursor, Repository, SortOrder,
+        f, field, mongo::Cursor, operator::*, pipeline, sync_indexes, BulkUpdate, BulkUpdateResult,
+        BulkUpdateUpsertResult, CollectionConfig, Index, IndexOption, Indexes, Model, ModelCursor,
+        Repository, SortOrder,
     };
     #[doc(no_inline)]
     pub use futures_util::future::{BoxFuture, FutureExt};
